from azure.core.pipeline.policies import SansIOHTTPPolicy


class ApiVersionAssertPolicy(SansIOHTTPPolicy):
    """
    Assert the ApiVersion is set properly on the response
    """

    def __init__(self, api_version):
        self.api_version = api_version

    def on_request(self, request):
        assert request.http_request.headers["x-ms-version"] == self.api_version



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.applicationinsights import ApplicationInsightsManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApplicationInsightsManagementProactiveDetectionConfigurationsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApplicationInsightsManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proactive_detection_configurations_list(self, resource_group):
        response = self.client.proactive_detection_configurations.list(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2015-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proactive_detection_configurations_get(self, resource_group):
        response = self.client.proactive_detection_configurations.get(
            resource_group_name=resource_group.name,
            resource_name="str",
            configuration_id="str",
            api_version="2015-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proactive_detection_configurations_update(self, resource_group):
        response = self.client.proactive_detection_configurations.update(
            resource_group_name=resource_group.name,
            resource_name="str",
            configuration_id="str",
            proactive_detection_properties={
                "customEmails": ["str"],
                "enabled": bool,
                "lastUpdatedTime": "str",
                "name": "str",
                "ruleDefinitions": {
                    "Description": "str",
                    "DisplayName": "str",
                    "HelpUrl": "str",
                    "IsEnabledByDefault": bool,
                    "IsHidden": bool,
                    "IsInPreview": bool,
                    "Name": "str",
                    "SupportsEmailNotifications": bool,
                },
                "sendEmailsToSubscriptionOwners": bool,
            },
            api_version="2015-05-01",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import SimpleModelValueOperations  # type: ignore
from ._operations import ModelWithRenamedPropertyValueOperations  # type: ignore
from ._operations import ModelWithRenamedFieldsValueOperations  # type: ignore
from ._operations import ModelWithNestedModelValueOperations  # type: ignore
from ._operations import ModelWithRenamedNestedModelValueOperations  # type: ignore
from ._operations import ModelWithSimpleArraysValueOperations  # type: ignore
from ._operations import ModelWithUnwrappedArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedArraysValueOperations  # type: ignore
from ._operations import ModelWithWrappedPrimitiveCustomItemNamesValueOperations  # type: ignore
from ._operations import ModelWithArrayOfModelValueOperations  # type: ignore
from ._operations import ModelWithUnwrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedWrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedUnwrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedWrappedAndItemModelArrayValueOperations  # type: ignore
from ._operations import ModelWithAttributesValueOperations  # type: ignore
from ._operations import ModelWithRenamedAttributeValueOperations  # type: ignore
from ._operations import ModelWithNamespaceValueOperations  # type: ignore
from ._operations import ModelWithNamespaceOnPropertiesValueOperations  # type: ignore
from ._operations import ModelWithTextValueOperations  # type: ignore
from ._operations import ModelWithOptionalFieldValueOperations  # type: ignore
from ._operations import ModelWithEmptyArrayValueOperations  # type: ignore
from ._operations import ModelWithDictionaryValueOperations  # type: ignore
from ._operations import ModelWithEncodedNamesValueOperations  # type: ignore
from ._operations import ModelWithEnumValueOperations  # type: ignore
from ._operations import ModelWithDatetimeValueOperations  # type: ignore
from ._operations import XmlErrorValueOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "SimpleModelValueOperations",
    "ModelWithRenamedPropertyValueOperations",
    "ModelWithRenamedFieldsValueOperations",
    "ModelWithNestedModelValueOperations",
    "ModelWithRenamedNestedModelValueOperations",
    "ModelWithSimpleArraysValueOperations",
    "ModelWithUnwrappedArrayValueOperations",
    "ModelWithRenamedArraysValueOperations",
    "ModelWithWrappedPrimitiveCustomItemNamesValueOperations",
    "ModelWithArrayOfModelValueOperations",
    "ModelWithUnwrappedModelArrayValueOperations",
    "ModelWithRenamedWrappedModelArrayValueOperations",
    "ModelWithRenamedUnwrappedModelArrayValueOperations",
    "ModelWithRenamedWrappedAndItemModelArrayValueOperations",
    "ModelWithAttributesValueOperations",
    "ModelWithRenamedAttributeValueOperations",
    "ModelWithNamespaceValueOperations",
    "ModelWithNamespaceOnPropertiesValueOperations",
    "ModelWithTextValueOperations",
    "ModelWithOptionalFieldValueOperations",
    "ModelWithEmptyArrayValueOperations",
    "ModelWithDictionaryValueOperations",
    "ModelWithEncodedNamesValueOperations",
    "ModelWithEnumValueOperations",
    "ModelWithDatetimeValueOperations",
    "XmlErrorValueOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# ---------------------------------------------------------
"""Foundry toolbox MCP bridge and discovery.

Centralises all toolbox-related logic:

- Discovering MCP server configs from ``mcp.json``
- Connecting to toolbox MCP endpoints via HTTP (JSON-RPC)
- Discovering available tools via ``tools/list``
- Creating Copilot SDK ``Tool`` wrappers that proxy calls to the MCP endpoint
- Handling auth token refresh

Instead of passing ``mcp_servers`` to the Copilot SDK (which has issues
with dotted tool names), we own the entire MCP interaction and register
tools as regular Copilot SDK custom tools.
"""
import asyncio
import json
import logging
import pathlib
import re
from typing import Any, Dict, List, Optional

import httpx
from copilot.tools import Tool, ToolResult

logger = logging.getLogger("azure.ai.agentserver.githubcopilot")

# Canary — proves which version of _toolbox.py is deployed.
_TOOLBOX_BUILD_TAG = "toolbox-v4-persistent-client"
logger.info("Toolbox module loaded: %s", _TOOLBOX_BUILD_TAG)

_FOUNDRY_TOOLBOX_FEATURE_HEADER = "Toolboxes=V1Preview"
_FOUNDRY_TOOLBOX_SERVER_KEY = "foundry-toolbox"
_FOUNDRY_SCOPE = "https://ai.azure.com/.default"



# ---------------------------------------------------------------------------
# Discovery — read mcp.json and build server config dicts
# ---------------------------------------------------------------------------

def discover_mcp_servers(
    project_root: pathlib.Path,
    toolbox_endpoint: Optional[str] = None,
) -> Dict[str, Any]:
    """Discover MCP server configs from ``mcp.json`` and an optional toolbox endpoint.

    Sources (merged in order, later wins on key collision):

    1. ``mcp.json`` in the project root — static config checked into the repo.
    2. *toolbox_endpoint* — an explicit URL for the Foundry toolbox MCP
       server.  When provided and ``mcp.json`` does not already define a
       ``"foundry-toolbox"`` entry, one is added automatically with the
       required ``Foundry-Features`` header and ``_auto_auth`` marker.

    For servers without an explicit ``Authorization`` header, the adapter
    will inject a fresh token automatically before each session.  The
    ``_auto_auth`` flag on the ``headers`` dict opts the server in to this
    behaviour.

    :param project_root: Agent project root directory (contains ``mcp.json``).
    :param toolbox_endpoint: Optional Foundry toolbox MCP endpoint URL.
    :returns: A dict of ``{server_name: server_config}``.
    """
    servers: Dict[str, Any] = {}

    # 1. Load from mcp.json
    mcp_path = project_root / "mcp.json"
    if mcp_path.exists():
        try:
            with open(mcp_path) as f:
                servers.update(json.load(f))
            logger.info("Loaded MCP servers from mcp.json: %s", list(servers.keys()))
        except Exception:
            logger.warning("Failed to load mcp.json", exc_info=True)

    # 2. Explicit toolbox endpoint
    if toolbox_endpoint and _FOUNDRY_TOOLBOX_SERVER_KEY not in servers:
        servers[_FOUNDRY_TOOLBOX_SERVER_KEY] = {
            "type": "http",
            "url": toolbox_endpoint,
            "tools": ["*"],
            "headers": {
                "Foundry-Features": _FOUNDRY_TOOLBOX_FEATURE_HEADER,
                "_auto_auth": True,
            },
        }
        logger.info("Added toolbox MCP server: %s", toolbox_endpoint)

    # Auto-inject headers for toolbox endpoints
    for server in servers.values():
        if not isinstance(server, dict):
            continue
        headers = server.setdefault("headers", {})
        if "Authorization" not in headers:
            headers["_auto_auth"] = True
        url = server.get("url", "").strip()
        server["url"] = url
        if "/toolboxes/" in url and "Foundry-Features" not in headers:
            headers["Foundry-Features"] = _FOUNDRY_TOOLBOX_FEATURE_HEADER

    return servers


# ---------------------------------------------------------------------------
# Auth helpers
# ---------------------------------------------------------------------------

def refresh_mcp_auth(servers: Dict[str, Any], credential: Any) -> None:
    """Refresh ``Authorization`` headers on MCP servers that opted in to auto-auth.

    :param servers: The ``mcp_servers`` dict from session config (mutated in place).
    :param credential: An Azure credential with a ``get_token()`` method.
    """
    token = credential.get_token(_FOUNDRY_SCOPE).token
    for server in servers.values():
        if not isinstance(server, dict):
            continue
        headers = server.get("headers", {})
        if headers.get("_auto_auth"):
            headers["Authorization"] = f"Bearer {token}"


# ---------------------------------------------------------------------------
# Tool name sanitisation
# ---------------------------------------------------------------------------

def _sanitize_tool_name(name: str) -> str:
    """Make a tool name safe for the Copilot SDK / LLM function-call API.

    Replaces dots, hyphens, and other non-alphanumeric/underscore characters
    with underscores, then strips the common ``FoundryMCPServerpreview.``
    prefix to keep names short and readable.
    """
    # Strip the verbose server prefix
    name = re.sub(r"^FoundryMCPServerpreview\.", "", name)
    # Also strip any other "ServerName." prefix pattern
    name = re.sub(r"^[A-Za-z0-9]+\.", "", name)
    # Replace invalid chars with underscores
    return re.sub(r"[^a-zA-Z0-9_]", "_", name)


# ---------------------------------------------------------------------------
# MCP Bridge — HTTP JSON-RPC client for Foundry toolbox endpoints
# ---------------------------------------------------------------------------

class McpBridge:
    """HTTP-based MCP client that connects to a Foundry toolbox MCP endpoint.

    Handles the MCP lifecycle: ``initialize`` → ``notifications/initialized``
    → ``tools/list`` → ``tools/call``.

    When *credential* is provided, the ``Authorization`` header is refreshed
    automatically before each ``call_tool`` request.
    """

    def __init__(self, endpoint: str, headers: Dict[str, str], credential: Any = None):
        self._endpoint = endpoint
        self._headers = dict(headers)
        self._session_id: Optional[str] = None
        self._req_id = 0
        self._credential = credential
        # Single persistent client for connection affinity through load balancers.
        self._client = httpx.AsyncClient(timeout=60.0)

    def _next_id(self) -> int:
        self._req_id += 1
        return self._req_id

    def _request_headers(self) -> Dict[str, str]:
        headers = dict(self._headers)
        if self._credential:
            try:
                token = self._credential.get_token(_FOUNDRY_SCOPE).token
                headers["Authorization"] = f"Bearer {token}"
            except Exception:
                logger.warning("Failed to refresh token for MCP bridge", exc_info=True)
        if self._session_id:
            headers["mcp-session-id"] = self._session_id
        return headers

    async def initialize(self) -> str:
        """Send MCP ``initialize`` + ``notifications/initialized``.

        :returns: The server name from the MCP ``initialize`` response.
        """
        auth_method = "credential" if self._credential else "static-header"
        has_auth = "Authorization" in self._headers
        logger.info(
            "MCP initialize: endpoint=%r (len=%d) auth_method=%s has_auth=%s",
            self._endpoint, len(self._endpoint), auth_method, has_auth,
        )

        resp = await self._client.post(
            self._endpoint,
            headers=self._headers,
            json={
                "jsonrpc": "2.0",
                "id": self._next_id(),
                "method": "initialize",
                "params": {
                    "protocolVersion": "2025-03-26",
                    "capabilities": {},
                    "clientInfo": {
                        "name": "agentserver-toolbox-bridge",
                        "version": "1.0.0",
                    },
                },
            },
        )
        diag_keys = ("x-ms-request-id", "x-ms-client-request-id", "x-request-id", "apim-request-id")
        diag = {k: resp.headers[k] for k in diag_keys if k in resp.headers}
        logger.info(
            "MCP initialize response: status=%d diagnostics=%s",
            resp.status_code, diag,
        )
        resp.raise_for_status()
        data = resp.json()
        self._session_id = resp.headers.get("mcp-session-id")

        # Send initialized notification
        await self._client.post(
            self._endpoint,
            headers=self._request_headers(),
            json={"jsonrpc": "2.0", "method": "notifications/initialized"},
        )

        return data.get("result", {}).get("serverInfo", {}).get("name", "unknown")

    async def list_tools(self) -> List[Dict[str, Any]]:
        """Call ``tools/list`` and return the tools array."""
        req_headers = self._request_headers()
        auth_method = "credential" if self._credential else "static-header"
        has_auth = "Authorization" in req_headers

        resp = await self._client.post(
            self._endpoint,
            headers=req_headers,
            json={
                "jsonrpc": "2.0",
                "id": self._next_id(),
                "method": "tools/list",
                "params": {},
            },
        )
        # Capture diagnostic headers before raising
        diag_keys = ("x-ms-request-id", "x-ms-client-request-id", "x-request-id", "apim-request-id")
        diag = {k: resp.headers[k] for k in diag_keys if k in resp.headers}
        logger.info(
            "MCP tools/list response: status=%d auth_method=%s has_auth=%s "
            "session_id=%s diagnostics=%s",
            resp.status_code, auth_method, has_auth,
            self._session_id, diag,
        )
        resp.raise_for_status()
        data = resp.json()
        if "error" in data:
            logger.warning("MCP tools/list error: %s diagnostics=%s", data["error"], diag)
        tools = data.get("result", {}).get("tools", [])
        if not tools:
            logger.warning(
                "MCP tools/list returned 0 tools. url=%s "
                "Response keys: %s, result keys: %s, "
                "full_response=%s, all_response_headers=%s, "
                "diagnostics=%s",
                self._endpoint,
                list(data.keys()),
                list(data.get("result", {}).keys()),
                json.dumps(data),
                dict(resp.headers),
                diag,
            )
        return tools

    async def call_tool(self, name: str, arguments: Dict[str, Any]) -> str:
        """Call ``tools/call`` and return the text result.

        :param name: The original MCP tool name (not sanitised).
        :param arguments: Tool arguments dict.
        :returns: Formatted text result.
        """
        logger.info("MCP tools/call: %s args=%s", name, list(arguments.keys()))
        resp = await self._client.post(
            self._endpoint,
            headers=self._request_headers(),
            json={
                "jsonrpc": "2.0",
                "id": self._next_id(),
                "method": "tools/call",
                "params": {"name": name, "arguments": arguments},
            },
        )
        resp.raise_for_status()
        data = resp.json()
        if "error" in data:
            err = data["error"]
            logger.warning("MCP tools/call error for %s: %s", name, err)
            return f"Error: {err.get('message', err)}"
        result = data.get("result", {})
        return _format_tool_result(result)

    async def close(self) -> None:
        """Close the persistent HTTP client."""
        await self._client.aclose()


# ---------------------------------------------------------------------------
# Result formatting
# ---------------------------------------------------------------------------

def _format_tool_result(result: Dict[str, Any]) -> str:
    """Extract text from an MCP ``tools/call`` result."""
    content = result.get("content", [])
    texts = [
        c.get("text", "")
        for c in content
        if isinstance(c, dict) and c.get("type") == "text"
    ]
    base_text = "\n".join(t for t in texts if t).strip()

    # Append citation metadata when present (Azure AI Search pattern)
    citations = _extract_citations(result)
    if not citations:
        return base_text or json.dumps(result)

    lines = ["", "Sources:"]
    for idx, c in enumerate(citations, start=1):
        title = c.get("title", "source")
        url = c.get("url", "")
        score = c.get("score")
        if score is not None:
            lines.append(f"{idx}. {title} (score: {score})")
        else:
            lines.append(f"{idx}. {title}")
        if url:
            lines.append(f"   {url}")

    citation_block = "\n".join(lines)
    if base_text:
        return f"{base_text}\n{citation_block}"
    return citation_block.lstrip()


def _extract_citations(result: Dict[str, Any]) -> List[Dict[str, Any]]:
    """Extract citation metadata from ``structuredContent.documents[]``."""
    structured = result.get("structuredContent")
    if not isinstance(structured, dict):
        return []
    docs = structured.get("documents")
    if not isinstance(docs, list):
        return []

    citations: List[Dict[str, Any]] = []
    for doc in docs:
        if not isinstance(doc, dict):
            continue
        citations.append({
            "title": doc.get("title") or doc.get("id") or "source",
            "url": doc.get("url"),
            "score": doc.get("score"),
        })
    return citations


# ---------------------------------------------------------------------------
# Copilot SDK Tool wrappers
# ---------------------------------------------------------------------------

def _make_copilot_tools(bridge: McpBridge, mcp_tools: List[Dict[str, Any]]) -> List[Tool]:
    """Convert MCP tool definitions into Copilot SDK ``Tool`` objects.

    Tool names are sanitised (stripped of server prefixes, dots/hyphens →
    underscores) because the Copilot API rejects names with those characters.
    The original MCP name is preserved for the ``tools/call`` RPC.
    """
    tools = []
    seen_names: Dict[str, int] = {}

    for mcp_tool in mcp_tools:
        mcp_name = mcp_tool["name"]
        sdk_name = _sanitize_tool_name(mcp_name)

        # Deduplicate names (e.g. two servers with same tool base name)
        if sdk_name in seen_names:
            seen_names[sdk_name] += 1
            sdk_name = f"{sdk_name}_{seen_names[sdk_name]}"
        else:
            seen_names[sdk_name] = 0

        desc = mcp_tool.get("description", f"MCP tool: {mcp_name}")
        schema = mcp_tool.get("inputSchema", {"type": "object", "properties": {}})

        def _make_handler(original_name: str):
            async def async_handler(invocation):
                args = getattr(invocation, "arguments", None) or {}
                if not isinstance(args, dict):
                    args = {}
                try:
                    result_text = await bridge.call_tool(original_name, args)
                    return ToolResult(text_result_for_llm=result_text)
                except Exception as e:
                    logger.warning("Tool %s failed: %s", original_name, e)
                    return ToolResult(
                        text_result_for_llm=f"Error calling {original_name}: {e}",
                        result_type="failure",
                        error=str(e),
                    )

            return async_handler

        tools.append(Tool(
            name=sdk_name,
            description=desc,
            parameters=schema,
            handler=_make_handler(mcp_name),
        ))

    return tools


# ---------------------------------------------------------------------------
# High-level: connect to toolbox and return SDK tools
# ---------------------------------------------------------------------------

async def connect_toolbox(
    endpoint: str,
    headers: Optional[Dict[str, str]] = None,
    credential: Any = None,
    name: Optional[str] = None,
) -> tuple["McpBridge", List[Tool]]:
    """Connect to a Foundry toolbox MCP endpoint and return SDK-ready tools.

    1. Builds auth headers from *headers* or *credential*
    2. Initialises the MCP session
    3. Discovers tools via ``tools/list``
    4. Creates Copilot SDK ``Tool`` wrappers

    :param endpoint: Toolbox MCP endpoint URL.
    :param headers: Pre-built headers dict (including Authorization).
    :param credential: Azure credential with ``get_token()`` method (used
        when *headers* is not provided or has no Authorization).
    :param name: Display name for logging.
    :returns: ``(bridge, tools)`` — caller must ``await bridge.close()`` on shutdown.
    """
    h = dict(headers or {})
    h.setdefault("Content-Type", "application/json")
    endpoint = endpoint.strip()

    # Auto-inject auth from credential if not already set
    if "Authorization" not in h and credential is not None:
        token = credential.get_token(_FOUNDRY_SCOPE).token
        h["Authorization"] = f"Bearer {token}"

    # Auto-inject toolbox feature header
    if "/toolboxes/" in endpoint:
        h.setdefault("Foundry-Features", _FOUNDRY_TOOLBOX_FEATURE_HEADER)

    bridge = McpBridge(endpoint, h, credential=credential)
    server_name = await bridge.initialize()
    mcp_tools = await bridge.list_tools()
    sdk_tools = _make_copilot_tools(bridge, mcp_tools)

    display = name or server_name
    logger.info(
        "Toolbox '%s' connected: %d tools discovered (%s)",
        display,
        len(sdk_tools),
        ", ".join(t.name for t in sdk_tools[:10])
        + ("..." if len(sdk_tools) > 10 else ""),
    )

    return bridge, sdk_tools



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import CollectionFormatClientTestBase, CollectionFormatPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCollectionFormatQueryOperations(CollectionFormatClientTestBase):
    @CollectionFormatPreparer()
    @recorded_by_proxy
    def test_query_multi(self, collectionformat_endpoint):
        client = self.create_client(endpoint=collectionformat_endpoint)
        response = client.query.multi(
            colors=["str"],
        )

        # please add some check logic here by yourself
        # ...

    @CollectionFormatPreparer()
    @recorded_by_proxy
    def test_query_ssv(self, collectionformat_endpoint):
        client = self.create_client(endpoint=collectionformat_endpoint)
        response = client.query.ssv(
            colors=["str"],
        )

        # please add some check logic here by yourself
        # ...

    @CollectionFormatPreparer()
    @recorded_by_proxy
    def test_query_pipes(self, collectionformat_endpoint):
        client = self.create_client(endpoint=collectionformat_endpoint)
        response = client.query.pipes(
            colors=["str"],
        )

        # please add some check logic here by yourself
        # ...

    @CollectionFormatPreparer()
    @recorded_by_proxy
    def test_query_csv(self, collectionformat_endpoint):
        client = self.create_client(endpoint=collectionformat_endpoint)
        response = client.query.csv(
            colors=["str"],
        )

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appnetwork import AppNetworkMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appnetwork
# USAGE
    python app_link_members_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AppNetworkMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.app_link_members.begin_create_or_update(
        resource_group_name="test_rg",
        app_link_name="applink-test-01",
        app_link_member_name="member-01",
        resource={
            "location": "westus2",
            "properties": {
                "clusterType": "AKS",
                "connectivityProfile": {
                    "eastWestGateway": {"visibility": "Internal"},
                    "privateConnect": {
                        "subnetResourceId": "/subscriptions/bc7e0da9-5e4c-4a91-9252-9658837006cf/resourceGroups/applink-vnet-rg/providers/Microsoft.Network/virtualNetworks/vnet1/subnets/subnet1"
                    },
                },
                "metadata": {
                    "resourceId": "/subscriptions/bc7e0da9-5e4c-4a91-9252-9658837006cf/resourcegroups/applink-rg/providers/Microsoft.ContainerService/managedClusters/applink-member1"
                },
                "upgradeProfile": {"fullyManagedUpgradeProfile": {"releaseChannel": "Stable"}, "mode": "FullyManaged"},
            },
            "tags": {"key2913": "test_tag"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-08-01-preview/AppLinkMembers_CreateOrUpdate.json
if __name__ == "__main__":
    main()



#!/usr/bin/env python

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

from setuptools import setup
from io import open

setup(
    name="azure-keyvault",
    version="4.2.1b1",
    description="Microsoft Azure Key Vault Client Libraries for Python",
    long_description_content_type="text/markdown",
    license="MIT License",
    author="Microsoft Corporation",
    author_email="azurekeyvault@microsoft.com",
    url="https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/keyvault/azure-keyvault",
    keywords="azure, azure sdk",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
    ],
    zip_safe=False,
    install_requires=[
        "azure-keyvault-certificates~=4.4",
        "azure-keyvault-secrets~=4.4",
        "azure-keyvault-keys~=4.5",
    ],
)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import NullableClientTestBase, NullablePreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestNullableCollectionsStringOperations(NullableClientTestBase):
    @NullablePreparer()
    @recorded_by_proxy
    def test_collections_string_get_non_null(self, nullable_endpoint):
        client = self.create_client(endpoint=nullable_endpoint)
        response = client.collections_string.get_non_null()

        # please add some check logic here by yourself
        # ...

    @NullablePreparer()
    @recorded_by_proxy
    def test_collections_string_get_null(self, nullable_endpoint):
        client = self.create_client(endpoint=nullable_endpoint)
        response = client.collections_string.get_null()

        # please add some check logic here by yourself
        # ...

    @NullablePreparer()
    @recorded_by_proxy
    def test_collections_string_patch_non_null(self, nullable_endpoint):
        client = self.create_client(endpoint=nullable_endpoint)
        response = client.collections_string.patch_non_null(
            body={"nullableProperty": ["str"], "requiredProperty": "str"},
        )

        # please add some check logic here by yourself
        # ...

    @NullablePreparer()
    @recorded_by_proxy
    def test_collections_string_patch_null(self, nullable_endpoint):
        client = self.create_client(endpoint=nullable_endpoint)
        response = client.collections_string.patch_null(
            body={"nullableProperty": ["str"], "requiredProperty": "str"},
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8

from enum import Enum
from corehttp.utils import CaseInsensitiveEnumMeta


class DogKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """extensible enum type for discriminator."""

    GOLDEN = "golden"
    """Species golden."""


class SnakeKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """fixed enum type for discriminator."""

    COBRA = "cobra"
    """Species cobra."""



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, Optional, TYPE_CHECKING, Union, overload

from .._utils.model_base import Model as _Model, rest_field

if TYPE_CHECKING:
    from .. import models as _models


class ErrorAdditionalInfo(_Model):
    """The resource management error additional info.

    :ivar type: The additional info type.
    :vartype type: str
    :ivar info: The additional info.
    :vartype info: any
    """

    type: Optional[str] = rest_field(visibility=["read"])
    """The additional info type."""
    info: Optional[Any] = rest_field(visibility=["read"])
    """The additional info."""


class ErrorDetail(_Model):
    """The error detail.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar target: The error target.
    :vartype target: str
    :ivar details: The error details.
    :vartype details: list[~azure.resourcemanager.resources.models.ErrorDetail]
    :ivar additional_info: The error additional info.
    :vartype additional_info: list[~azure.resourcemanager.resources.models.ErrorAdditionalInfo]
    """

    code: Optional[str] = rest_field(visibility=["read"])
    """The error code."""
    message: Optional[str] = rest_field(visibility=["read"])
    """The error message."""
    target: Optional[str] = rest_field(visibility=["read"])
    """The error target."""
    details: Optional[list["_models.ErrorDetail"]] = rest_field(visibility=["read"])
    """The error details."""
    additional_info: Optional[list["_models.ErrorAdditionalInfo"]] = rest_field(
        name="additionalInfo", visibility=["read"]
    )
    """The error additional info."""


class ErrorResponse(_Model):
    """Error response.

    :ivar error: The error object.
    :vartype error: ~azure.resourcemanager.resources.models.ErrorDetail
    """

    error: Optional["_models.ErrorDetail"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The error object."""

    @overload
    def __init__(
        self,
        *,
        error: Optional["_models.ErrorDetail"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Resource(_Model):
    """Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    """

    id: Optional[str] = rest_field(visibility=["read"])
    """Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}."""
    name: Optional[str] = rest_field(visibility=["read"])
    """The name of the resource."""
    type: Optional[str] = rest_field(visibility=["read"])
    """The type of the resource. E.g. \"Microsoft.Compute/virtualMachines\" or
     \"Microsoft.Storage/storageAccounts\"."""
    system_data: Optional["_models.SystemData"] = rest_field(name="systemData", visibility=["read"])
    """Azure Resource Manager metadata containing createdBy and modifiedBy information."""


class ExtensionResource(Resource):
    """The base extension resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    """


class ExtensionsResource(ExtensionResource):
    """Concrete extension resource types can be created by aliasing this type using a specific
    property type.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.resourcemanager.resources.models.ExtensionsResourceProperties
    """

    properties: Optional["_models.ExtensionsResourceProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.ExtensionsResourceProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ExtensionsResourceProperties(_Model):
    """ExtensionsResource properties.

    :ivar description: The description of the resource.
    :vartype description: str
    :ivar provisioning_state: The status of the last operation. Known values are: "Succeeded",
     "Failed", "Canceled", "Provisioning", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.resourcemanager.resources.models.ProvisioningState
    """

    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The description of the resource."""
    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """The status of the last operation. Known values are: \"Succeeded\", \"Failed\", \"Canceled\",
     \"Provisioning\", \"Updating\", \"Deleting\", and \"Accepted\"."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ProxyResource(Resource):
    """Proxy Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    """


class LocationResource(ProxyResource):
    """Concrete proxy resource types can be created by aliasing this type using a specific property
    type.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.resourcemanager.resources.models.LocationResourceProperties
    """

    properties: Optional["_models.LocationResourceProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.LocationResourceProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class LocationResourceProperties(_Model):
    """Location resource properties.

    :ivar description: The description of the resource.
    :vartype description: str
    :ivar provisioning_state: The status of the last operation. Known values are: "Succeeded",
     "Failed", "Canceled", "Provisioning", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.resourcemanager.resources.models.ProvisioningState
    """

    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The description of the resource."""
    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """The status of the last operation. Known values are: \"Succeeded\", \"Failed\", \"Canceled\",
     \"Provisioning\", \"Updating\", \"Deleting\", and \"Accepted\"."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class NestedProxyResource(ProxyResource):
    """Nested child of Top Level Tracked Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.resourcemanager.resources.models.NestedProxyResourceProperties
    """

    properties: Optional["_models.NestedProxyResourceProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.NestedProxyResourceProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class NestedProxyResourceProperties(_Model):
    """Nested Proxy Resource Properties.

    :ivar provisioning_state: Provisioning State of the nested child Resource. Known values are:
     "Succeeded", "Failed", "Canceled", "Provisioning", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.resourcemanager.resources.models.ProvisioningState
    :ivar description: Nested resource description.
    :vartype description: str
    """

    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """Provisioning State of the nested child Resource. Known values are: \"Succeeded\", \"Failed\",
     \"Canceled\", \"Provisioning\", \"Updating\", \"Deleting\", and \"Accepted\"."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Nested resource description."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class NotificationDetails(_Model):
    """The details of a user notification.

    :ivar message: The notification message. Required.
    :vartype message: str
    :ivar urgent: If true, the notification is urgent. Required.
    :vartype urgent: bool
    """

    message: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The notification message. Required."""
    urgent: bool = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """If true, the notification is urgent. Required."""

    @overload
    def __init__(
        self,
        *,
        message: str,
        urgent: bool,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class TrackedResource(Resource):
    """Tracked Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar location: The geo-location where the resource lives. Required.
    :vartype location: str
    """

    tags: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Resource tags."""
    location: str = rest_field(visibility=["read", "create"])
    """The geo-location where the resource lives. Required."""

    @overload
    def __init__(
        self,
        *,
        location: str,
        tags: Optional[dict[str, str]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SingletonTrackedResource(TrackedResource):
    """Concrete tracked resource types can be created by aliasing this type using a specific property
    type.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar location: The geo-location where the resource lives. Required.
    :vartype location: str
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.resourcemanager.resources.models.SingletonTrackedResourceProperties
    """

    properties: Optional["_models.SingletonTrackedResourceProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        location: str,
        tags: Optional[dict[str, str]] = None,
        properties: Optional["_models.SingletonTrackedResourceProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SingletonTrackedResourceProperties(_Model):
    """Singleton Arm Resource Properties.

    :ivar provisioning_state: The status of the last operation. Known values are: "Succeeded",
     "Failed", "Canceled", "Provisioning", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.resourcemanager.resources.models.ProvisioningState
    :ivar description: The description of the resource.
    :vartype description: str
    """

    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """The status of the last operation. Known values are: \"Succeeded\", \"Failed\", \"Canceled\",
     \"Provisioning\", \"Updating\", \"Deleting\", and \"Accepted\"."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The description of the resource."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SystemData(_Model):
    """Metadata pertaining to creation and last modification of the resource.

    :ivar created_by: The identity that created the resource.
    :vartype created_by: str
    :ivar created_by_type: The type of identity that created the resource. Known values are:
     "User", "Application", "ManagedIdentity", and "Key".
    :vartype created_by_type: str or ~azure.resourcemanager.resources.models.CreatedByType
    :ivar created_at: The timestamp of resource creation (UTC).
    :vartype created_at: ~datetime.datetime
    :ivar last_modified_by: The identity that last modified the resource.
    :vartype last_modified_by: str
    :ivar last_modified_by_type: The type of identity that last modified the resource. Known values
     are: "User", "Application", "ManagedIdentity", and "Key".
    :vartype last_modified_by_type: str or ~azure.resourcemanager.resources.models.CreatedByType
    :ivar last_modified_at: The timestamp of resource last modification (UTC).
    :vartype last_modified_at: ~datetime.datetime
    """

    created_by: Optional[str] = rest_field(name="createdBy", visibility=["read", "create", "update", "delete", "query"])
    """The identity that created the resource."""
    created_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="createdByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that created the resource. Known values are: \"User\", \"Application\",
     \"ManagedIdentity\", and \"Key\"."""
    created_at: Optional[datetime.datetime] = rest_field(
        name="createdAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource creation (UTC)."""
    last_modified_by: Optional[str] = rest_field(
        name="lastModifiedBy", visibility=["read", "create", "update", "delete", "query"]
    )
    """The identity that last modified the resource."""
    last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="lastModifiedByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that last modified the resource. Known values are: \"User\",
     \"Application\", \"ManagedIdentity\", and \"Key\"."""
    last_modified_at: Optional[datetime.datetime] = rest_field(
        name="lastModifiedAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource last modification (UTC)."""

    @overload
    def __init__(
        self,
        *,
        created_by: Optional[str] = None,
        created_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        created_at: Optional[datetime.datetime] = None,
        last_modified_by: Optional[str] = None,
        last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        last_modified_at: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class TopLevelTrackedResource(TrackedResource):
    """Concrete tracked resource types can be created by aliasing this type using a specific property
    type.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.resourcemanager.resources.models.SystemData
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar location: The geo-location where the resource lives. Required.
    :vartype location: str
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.resourcemanager.resources.models.TopLevelTrackedResourceProperties
    """

    properties: Optional["_models.TopLevelTrackedResourceProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        location: str,
        tags: Optional[dict[str, str]] = None,
        properties: Optional["_models.TopLevelTrackedResourceProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class TopLevelTrackedResourceProperties(_Model):
    """Top Level Arm Resource Properties.

    :ivar provisioning_state: The status of the last operation. Known values are: "Succeeded",
     "Failed", "Canceled", "Provisioning", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.resourcemanager.resources.models.ProvisioningState
    :ivar description: The description of the resource.
    :vartype description: str
    """

    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """The status of the last operation. Known values are: \"Succeeded\", \"Failed\", \"Canceled\",
     \"Provisioning\", \"Updating\", \"Deleting\", and \"Accepted\"."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The description of the resource."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# coding=utf-8

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient, policies

from .._utils.serialization import Deserializer, Serializer
from ._configuration import UnionClientConfiguration
from .operations import (
    EnumsOnlyOperations,
    FloatsOnlyOperations,
    IntsOnlyOperations,
    MixedLiteralsOperations,
    MixedTypesOperations,
    ModelsOnlyOperations,
    StringAndArrayOperations,
    StringExtensibleNamedOperations,
    StringExtensibleOperations,
    StringsOnlyOperations,
)


class UnionClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Describe scenarios for various combinations of unions.

    :ivar strings_only: StringsOnlyOperations operations
    :vartype strings_only: typetest.union.aio.operations.StringsOnlyOperations
    :ivar string_extensible: StringExtensibleOperations operations
    :vartype string_extensible: typetest.union.aio.operations.StringExtensibleOperations
    :ivar string_extensible_named: StringExtensibleNamedOperations operations
    :vartype string_extensible_named: typetest.union.aio.operations.StringExtensibleNamedOperations
    :ivar ints_only: IntsOnlyOperations operations
    :vartype ints_only: typetest.union.aio.operations.IntsOnlyOperations
    :ivar floats_only: FloatsOnlyOperations operations
    :vartype floats_only: typetest.union.aio.operations.FloatsOnlyOperations
    :ivar models_only: ModelsOnlyOperations operations
    :vartype models_only: typetest.union.aio.operations.ModelsOnlyOperations
    :ivar enums_only: EnumsOnlyOperations operations
    :vartype enums_only: typetest.union.aio.operations.EnumsOnlyOperations
    :ivar string_and_array: StringAndArrayOperations operations
    :vartype string_and_array: typetest.union.aio.operations.StringAndArrayOperations
    :ivar mixed_literals: MixedLiteralsOperations operations
    :vartype mixed_literals: typetest.union.aio.operations.MixedLiteralsOperations
    :ivar mixed_types: MixedTypesOperations operations
    :vartype mixed_types: typetest.union.aio.operations.MixedTypesOperations
    :keyword endpoint: Service host. Default value is "http://localhost:3000".
    :paramtype endpoint: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, *, endpoint: str = "http://localhost:3000", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = UnionClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.strings_only = StringsOnlyOperations(self._client, self._config, self._serialize, self._deserialize)
        self.string_extensible = StringExtensibleOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.string_extensible_named = StringExtensibleNamedOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.ints_only = IntsOnlyOperations(self._client, self._config, self._serialize, self._deserialize)
        self.floats_only = FloatsOnlyOperations(self._client, self._config, self._serialize, self._deserialize)
        self.models_only = ModelsOnlyOperations(self._client, self._config, self._serialize, self._deserialize)
        self.enums_only = EnumsOnlyOperations(self._client, self._config, self._serialize, self._deserialize)
        self.string_and_array = StringAndArrayOperations(self._client, self._config, self._serialize, self._deserialize)
        self.mixed_literals = MixedLiteralsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.mixed_types = MixedTypesOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload
import urllib.parse

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._configuration import MethodSubscriptionIdClientConfiguration
from .._utils.model_base import SdkJSONEncoder, _deserialize, _failsafe_deserialize
from .._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]
JSON = MutableMapping[str, Any]
List = list

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_operations_list_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/providers/Azure.ResourceManager.MethodSubscriptionId/operations"

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource1_operations_get_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource1_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource1s/{subscriptionResource1Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource1Name": _SERIALIZER.url("subscription_resource1_name", subscription_resource1_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource1_operations_put_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource1_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource1s/{subscriptionResource1Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource1Name": _SERIALIZER.url("subscription_resource1_name", subscription_resource1_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource1_operations_delete_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource1_name: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource1s/{subscriptionResource1Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource1Name": _SERIALIZER.url("subscription_resource1_name", subscription_resource1_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource2_operations_get_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource2_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource2s/{subscriptionResource2Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource2Name": _SERIALIZER.url("subscription_resource2_name", subscription_resource2_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource2_operations_put_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource2_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource2s/{subscriptionResource2Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource2Name": _SERIALIZER.url("subscription_resource2_name", subscription_resource2_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_two_subscription_resources_method_level_subscription_resource2_operations_delete_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource2_name: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResource2s/{subscriptionResource2Name}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResource2Name": _SERIALIZER.url("subscription_resource2_name", subscription_resource2_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


def build_mixed_subscription_placement_subscription_resource_operations_get_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResources/{subscriptionResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResourceName": _SERIALIZER.url("subscription_resource_name", subscription_resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_subscription_placement_subscription_resource_operations_put_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResources/{subscriptionResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResourceName": _SERIALIZER.url("subscription_resource_name", subscription_resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_subscription_placement_subscription_resource_operations_delete_request(  # pylint: disable=name-too-long
    subscription_id: str, subscription_resource_name: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    # Construct URL
    _url = "/subscriptions/{subscriptionId}/providers/Azure.ResourceManager.MethodSubscriptionId/subscriptionResources/{subscriptionResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "subscriptionResourceName": _SERIALIZER.url("subscription_resource_name", subscription_resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


def build_mixed_subscription_placement_resource_group_resource_operations_get_request(  # pylint: disable=name-too-long
    resource_group_name: str, resource_group_resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Azure.ResourceManager.MethodSubscriptionId/resourceGroupResources/{resourceGroupResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "resourceGroupResourceName": _SERIALIZER.url(
            "resource_group_resource_name", resource_group_resource_name, "str"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_subscription_placement_resource_group_resource_operations_put_request(  # pylint: disable=name-too-long
    resource_group_name: str, resource_group_resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Azure.ResourceManager.MethodSubscriptionId/resourceGroupResources/{resourceGroupResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "resourceGroupResourceName": _SERIALIZER.url(
            "resource_group_resource_name", resource_group_resource_name, "str"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_subscription_placement_resource_group_resource_operations_delete_request(  # pylint: disable=name-too-long
    resource_group_name: str, resource_group_resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    # Construct URL
    _url = "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Azure.ResourceManager.MethodSubscriptionId/resourceGroupResources/{resourceGroupResourceName}"
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "resourceGroupResourceName": _SERIALIZER.url(
            "resource_group_resource_name", resource_group_resource_name, "str"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


class TwoSubscriptionResourcesMethodLevelOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`two_subscription_resources_method_level` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

        self.subscription_resource1_operations = (
            TwoSubscriptionResourcesMethodLevelSubscriptionResource1OperationsOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )
        self.subscription_resource2_operations = (
            TwoSubscriptionResourcesMethodLevelSubscriptionResource2OperationsOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )


class MixedSubscriptionPlacementOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`mixed_subscription_placement` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

        self.subscription_resource_operations = MixedSubscriptionPlacementSubscriptionResourceOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.resource_group_resource_operations = MixedSubscriptionPlacementResourceGroupResourceOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )


class Operations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`operations` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, **kwargs: Any) -> ItemPaged["_models.Operation"]:
        """List the operations for the provider.

        :return: An iterator like instance of Operation
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.resourcemanager.methodsubscriptionid.models.Operation]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[List[_models.Operation]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_operations_list_request(
                    api_version=self._config.api_version,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.base_url", self._config.base_url, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET",
                    urllib.parse.urljoin(next_link, _parsed_next_link.path),
                    headers=_headers,
                    params=_next_request_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.base_url", self._config.base_url, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                List[_models.Operation],
                deserialized.get("value", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextLink") or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = _failsafe_deserialize(
                    _models.ErrorResponse,
                    response,
                )
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)


class TwoSubscriptionResourcesMethodLevelSubscriptionResource1OperationsOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`subscription_resource1_operations` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get(
        self, subscription_id: str, subscription_resource1_name: str, **kwargs: Any
    ) -> _models.SubscriptionResource1:
        """Get a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :return: SubscriptionResource1. The SubscriptionResource1 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.SubscriptionResource1] = kwargs.pop("cls", None)

        _request = build_two_subscription_resources_method_level_subscription_resource1_operations_get_request(
            subscription_id=subscription_id,
            subscription_resource1_name=subscription_resource1_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource1, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource1_name: str,
        resource: _models.SubscriptionResource1,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource1:
        """Create a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :param resource: Resource create parameters. Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource1. The SubscriptionResource1 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource1_name: str,
        resource: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource1:
        """Create a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :param resource: Resource create parameters. Required.
        :type resource: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource1. The SubscriptionResource1 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource1_name: str,
        resource: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource1:
        """Create a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :param resource: Resource create parameters. Required.
        :type resource: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource1. The SubscriptionResource1 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def put(
        self,
        subscription_id: str,
        subscription_resource1_name: str,
        resource: Union[_models.SubscriptionResource1, JSON, IO[bytes]],
        **kwargs: Any
    ) -> _models.SubscriptionResource1:
        """Create a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :param resource: Resource create parameters. Is one of the following types:
         SubscriptionResource1, JSON, IO[bytes] Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1 or
         JSON or IO[bytes]
        :return: SubscriptionResource1. The SubscriptionResource1 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource1
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SubscriptionResource1] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(resource, (IOBase, bytes)):
            _content = resource
        else:
            _content = json.dumps(resource, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_two_subscription_resources_method_level_subscription_resource1_operations_put_request(
            subscription_id=subscription_id,
            subscription_resource1_name=subscription_resource1_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource1, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, subscription_id: str, subscription_resource1_name: str, **kwargs: Any
    ) -> None:
        """Delete a SubscriptionResource1.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource1_name: The name of the SubscriptionResource1. Required.
        :type subscription_resource1_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_two_subscription_resources_method_level_subscription_resource1_operations_delete_request(
            subscription_id=subscription_id,
            subscription_resource1_name=subscription_resource1_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class TwoSubscriptionResourcesMethodLevelSubscriptionResource2OperationsOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`subscription_resource2_operations` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get(
        self, subscription_id: str, subscription_resource2_name: str, **kwargs: Any
    ) -> _models.SubscriptionResource2:
        """Get a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :return: SubscriptionResource2. The SubscriptionResource2 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.SubscriptionResource2] = kwargs.pop("cls", None)

        _request = build_two_subscription_resources_method_level_subscription_resource2_operations_get_request(
            subscription_id=subscription_id,
            subscription_resource2_name=subscription_resource2_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource2, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource2_name: str,
        resource: _models.SubscriptionResource2,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource2:
        """Create a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :param resource: Resource create parameters. Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource2. The SubscriptionResource2 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource2_name: str,
        resource: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource2:
        """Create a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :param resource: Resource create parameters. Required.
        :type resource: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource2. The SubscriptionResource2 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource2_name: str,
        resource: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource2:
        """Create a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :param resource: Resource create parameters. Required.
        :type resource: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource2. The SubscriptionResource2 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def put(
        self,
        subscription_id: str,
        subscription_resource2_name: str,
        resource: Union[_models.SubscriptionResource2, JSON, IO[bytes]],
        **kwargs: Any
    ) -> _models.SubscriptionResource2:
        """Create a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :param resource: Resource create parameters. Is one of the following types:
         SubscriptionResource2, JSON, IO[bytes] Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2 or
         JSON or IO[bytes]
        :return: SubscriptionResource2. The SubscriptionResource2 is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource2
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SubscriptionResource2] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(resource, (IOBase, bytes)):
            _content = resource
        else:
            _content = json.dumps(resource, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_two_subscription_resources_method_level_subscription_resource2_operations_put_request(
            subscription_id=subscription_id,
            subscription_resource2_name=subscription_resource2_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource2, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, subscription_id: str, subscription_resource2_name: str, **kwargs: Any
    ) -> None:
        """Delete a SubscriptionResource2.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource2_name: The name of the SubscriptionResource2. Required.
        :type subscription_resource2_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_two_subscription_resources_method_level_subscription_resource2_operations_delete_request(
            subscription_id=subscription_id,
            subscription_resource2_name=subscription_resource2_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class MixedSubscriptionPlacementSubscriptionResourceOperationsOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`subscription_resource_operations` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get(self, subscription_id: str, subscription_resource_name: str, **kwargs: Any) -> _models.SubscriptionResource:
        """Get a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :return: SubscriptionResource. The SubscriptionResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.SubscriptionResource] = kwargs.pop("cls", None)

        _request = build_mixed_subscription_placement_subscription_resource_operations_get_request(
            subscription_id=subscription_id,
            subscription_resource_name=subscription_resource_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource_name: str,
        resource: _models.SubscriptionResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource:
        """Create a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource. The SubscriptionResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource_name: str,
        resource: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource:
        """Create a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource. The SubscriptionResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        subscription_id: str,
        subscription_resource_name: str,
        resource: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.SubscriptionResource:
        """Create a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SubscriptionResource. The SubscriptionResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def put(
        self,
        subscription_id: str,
        subscription_resource_name: str,
        resource: Union[_models.SubscriptionResource, JSON, IO[bytes]],
        **kwargs: Any
    ) -> _models.SubscriptionResource:
        """Create a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :param resource: Resource create parameters. Is one of the following types:
         SubscriptionResource, JSON, IO[bytes] Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource or JSON
         or IO[bytes]
        :return: SubscriptionResource. The SubscriptionResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.SubscriptionResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SubscriptionResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(resource, (IOBase, bytes)):
            _content = resource
        else:
            _content = json.dumps(resource, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_mixed_subscription_placement_subscription_resource_operations_put_request(
            subscription_id=subscription_id,
            subscription_resource_name=subscription_resource_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.SubscriptionResource, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, subscription_id: str, subscription_resource_name: str, **kwargs: Any
    ) -> None:
        """Delete a SubscriptionResource.

        :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
        :type subscription_id: str
        :param subscription_resource_name: The name of the SubscriptionResource. Required.
        :type subscription_resource_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_mixed_subscription_placement_subscription_resource_operations_delete_request(
            subscription_id=subscription_id,
            subscription_resource_name=subscription_resource_name,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class MixedSubscriptionPlacementResourceGroupResourceOperationsOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.resourcemanager.methodsubscriptionid.MethodSubscriptionIdClient`'s
        :attr:`resource_group_resource_operations` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: MethodSubscriptionIdClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get(
        self, resource_group_name: str, resource_group_resource_name: str, **kwargs: Any
    ) -> _models.ResourceGroupResource:
        """Get a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :return: ResourceGroupResource. The ResourceGroupResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.ResourceGroupResource] = kwargs.pop("cls", None)

        _request = build_mixed_subscription_placement_resource_group_resource_operations_get_request(
            resource_group_name=resource_group_name,
            resource_group_resource_name=resource_group_resource_name,
            subscription_id=self._config.subscription_id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.ResourceGroupResource, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def put(
        self,
        resource_group_name: str,
        resource_group_resource_name: str,
        resource: _models.ResourceGroupResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ResourceGroupResource:
        """Create a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ResourceGroupResource. The ResourceGroupResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        resource_group_name: str,
        resource_group_resource_name: str,
        resource: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ResourceGroupResource:
        """Create a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ResourceGroupResource. The ResourceGroupResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def put(
        self,
        resource_group_name: str,
        resource_group_resource_name: str,
        resource: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ResourceGroupResource:
        """Create a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :param resource: Resource create parameters. Required.
        :type resource: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ResourceGroupResource. The ResourceGroupResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def put(
        self,
        resource_group_name: str,
        resource_group_resource_name: str,
        resource: Union[_models.ResourceGroupResource, JSON, IO[bytes]],
        **kwargs: Any
    ) -> _models.ResourceGroupResource:
        """Create a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :param resource: Resource create parameters. Is one of the following types:
         ResourceGroupResource, JSON, IO[bytes] Required.
        :type resource: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource or
         JSON or IO[bytes]
        :return: ResourceGroupResource. The ResourceGroupResource is compatible with MutableMapping
        :rtype: ~azure.resourcemanager.methodsubscriptionid.models.ResourceGroupResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ResourceGroupResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(resource, (IOBase, bytes)):
            _content = resource
        else:
            _content = json.dumps(resource, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_mixed_subscription_placement_resource_group_resource_operations_put_request(
            resource_group_name=resource_group_name,
            resource_group_resource_name=resource_group_resource_name,
            subscription_id=self._config.subscription_id,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.ResourceGroupResource, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, resource_group_resource_name: str, **kwargs: Any
    ) -> None:
        """Delete a ResourceGroupResource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_group_resource_name: The name of the ResourceGroupResource. Required.
        :type resource_group_resource_name: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_mixed_subscription_placement_resource_group_resource_operations_delete_request(
            resource_group_name=resource_group_name,
            resource_group_resource_name=resource_group_resource_name,
            subscription_id=self._config.subscription_id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

from typing import Any, Mapping, Optional, TYPE_CHECKING, overload

from .._utils.model_base import Model as _Model, rest_field

if TYPE_CHECKING:
    from .. import models as _models


class ListWidgetsResponse(_Model):
    """ListWidgetsResponse.

    :ivar widgets: Required.
    :vartype widgets: list[~specs.azure.versioning.previewversion.models.Widget]
    """

    widgets: list["_models.Widget"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        widgets: list["_models.Widget"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateWidgetColorRequest(_Model):
    """Update widget color request.

    :ivar color: New color for the widget. Required.
    :vartype color: str
    """

    color: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """New color for the widget. Required."""

    @overload
    def __init__(
        self,
        *,
        color: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Widget(_Model):
    """A simple model for testing.

    :ivar id: Widget identifier. Required.
    :vartype id: str
    :ivar name: Widget name. Required.
    :vartype name: str
    :ivar color: Widget color, only available in preview version.
    :vartype color: str
    """

    id: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Widget identifier. Required."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Widget name. Required."""
    color: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Widget color, only available in preview version."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        name: str,
        color: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.authorization.aio import AuthorizationManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAuthorizationManagementAccessReviewDefaultSettingsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AuthorizationManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_access_review_default_settings_get(self, resource_group):
        response = await self.client.access_review_default_settings.get(
            api_version="2021-12-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_access_review_default_settings_put(self, resource_group):
        response = await self.client.access_review_default_settings.put(
            properties={
                "autoApplyDecisionsEnabled": bool,
                "defaultDecision": "str",
                "defaultDecisionEnabled": bool,
                "instanceDurationInDays": 0,
                "justificationRequiredOnApproval": bool,
                "mailNotificationsEnabled": bool,
                "recommendationLookBackDuration": "1 day, 0:00:00",
                "recommendationsEnabled": bool,
                "recurrence": {
                    "pattern": {"interval": 0, "type": "str"},
                    "range": {
                        "endDate": "2020-02-20 00:00:00",
                        "numberOfOccurrences": 0,
                        "startDate": "2020-02-20 00:00:00",
                        "type": "str",
                    },
                },
                "reminderNotificationsEnabled": bool,
            },
            api_version="2021-12-01-preview",
        )

        # please add some check logic here by yourself
        # ...
